'****************************************************************
'*  Name    : Desk Control Console - Led dimmer / Fan control   *
'*  Author  : Stijn Coenen [Stynus]                             *
'*  Notice  : Licenced under Creative Commons                   *
'*          : Attribution-Noncommercial 2.0 Belgium Licence     *
'*          : http://creativecommons.org/licenses/by-nc/2.0/be/ *
'*  Date    : 27/03/2011                                        *
'*  Version : 1.1                                               *
'****************************************************************
Device 16F688
Config FCMEN_ON, IESO_ON, BOD_ON, CPD_OFF, CP_OFF, MCLRE_OFF, PWRTE_OFF, WDT_OFF, INTRC_OSC_NOCLKOUT
Xtal = 4     'Internal oscillator frequency
All_Digital = true
'****************************************************************
'*                   Symbols & Variables                        *
'****************************************************************
Symbol LED1 = PORTC.2
Symbol LED2 = PORTC.1
Symbol Fan  = PORTC.0
TRISC = 0
TRISA = 255

Dim PWM_Counter As Byte   ' Used for the pwm loop
Dim LED1_Value  As Byte   ' The PWM value of LED 1
Dim LED2_Value  As Byte   ' The PWM value of LED 2
Dim Fan_Value   As Byte   ' The PWM value of fan
Dim LED1_AnIN   As Byte   ' The analog value of LED 1
Dim LED2_AnIN   As Byte   ' The analog value of LED 2
Dim Fan_AnIN    As Byte   ' The analog value of fan
Dim index       As Byte   ' Used for the soft start loop at startup

'AD converter
Declare Adin_Res = 8      ' 8-bit result 
Declare Adin_Tad = FRC    ' RC OSC chosen  
Declare Adin_Stime = 50   ' Allow 50us sample time  
 
Clear                     ' Clear all ram

GoTo start_up             ' Go to start_up
'****************************************************************
'*                         Subroutines                          *
'****************************************************************
PWM_Out:
    ' Increment PWM_Counter, if PWM_Counter is bigger than a desired 
    ' value then that output is turned off 
    For PWM_Counter = 0 To 255
        If PWM_Counter >= LED1_Value Then
            LED1 = 0
        Else
            LED1 = 1
        EndIf
        If PWM_Counter >= LED2_Value Then
            LED2 = 0
        Else
            LED2 = 1
        EndIf   
        If PWM_Counter >= Fan_Value Then
            Fan = 0
        Else
            Fan = 1
        EndIf    
        DelayUS 8
    Next 
Return
Analog_In:
    ' Read the analog inputs
    LED1_AnIN  = ADIn 2
    LED2_AnIN  = ADIn 1
    Fan_AnIN   = ADIn 0
Return
'****************************************************************
'*                         Start up                             *
'****************************************************************
start_up:
    ' Softstart leds + hardstart fan
    GoSub Analog_In       
    LED1_Value  = 0       ' LEDS soft start from 0
    LED2_Value  = 0
    'Fan value high enough?
    If Fan_AnIN < 32 Then
        Fan_Value   = 0   
    Else
        Fan_Value   = 255 ' Fan 100% for start up
    EndIf
    ' Softstart the led's
    For index   = 0 To 255 Step 4
        ' Increase the led value's as long as they remain under the 
        ' input value
        If index < LED1_AnIN Then
            LED1_Value = index
        EndIf
        If index < LED2_AnIN Then
            LED2_Value = index
        EndIf
        GoSub PWM_Out
    Next 
'****************************************************************
'*                         Main loop                            *
'****************************************************************
While 1 = 1                  ' Infinite loop
    GoSub Analog_In          ' Read the analog inputs
    
    LED1_Value = LED1_AnIN   ' Transfer the led values in the pwm values
    LED2_Value = LED2_AnIN
    ' If the fan value is too low, shut it down completely
    ' Otherwise the fan motor will stall
    If Fan_AnIN < 32 Then
        Fan_Value = 0
    Else
        Fan_Value  = Fan_AnIN
    EndIf
    
    GoSub PWM_Out            ' Output the pwm values
Wend
'****************************************************************
End
